@@ -17,18 +17,20 @@ import ctypes as ct
 import numpy as np
 from numpy.random import MT19937, Generator
 from scipy.optimize import OptimizeResult
-from fcmaes.cmaescpp import callback_par, call_back_par, freemem, libcmalib
-from fcmaes.cmaes import parallel, _check_bounds
+from fcmaes.ldecpp import callback_par, call_back_par
+from fcmaes.decpp import libcmalib
+from fcmaes.cmaes import _check_bounds
+from fcmaes.gcldecpp import parallel
 
 os.environ['MKL_DEBUG_CPU_TYPE'] = '5'
 
 def minimize(fun, 
-             bounds, 
+             bounds=None, 
              x0=None, 
              input_sigma = 0.3, 
              popsize = None, 
              max_evaluations = 100000, 
-             stop_fittness = None, 
+             stop_fitness = None, 
              pbest = 0.7,
              f0 = 0.0,
              cr0 = 0.0,
@@ -43,24 +45,22 @@ def minimize(fun,
     ----------
     fun : callable
         The objective function to be minimized.
-            ``fun(x, *args) -> float``
-        where ``x`` is an 1-D array with shape (n,) and ``args``
-        is a tuple of the fixed parameters needed to completely
-        specify the function.
+            ``fun(x) -> float``
+        where ``x`` is an 1-D array with shape (dim,)
     bounds : sequence or `Bounds`
         Bounds on variables. There are two ways to specify the bounds:
             1. Instance of the `scipy.Bounds` class.
             2. Sequence of ``(min, max)`` pairs for each element in `x`.
-    x0 : ndarray, shape (n,)
-        Initial guess. Array of real elements of size (n,),
-        where 'n' is the number of independent variables.  
-    input_sigma : ndarray, shape (n,) or scalar
+    x0 : ndarray, shape (dim,)
+        Initial guess. Array of real elements of size (dim,),
+        where 'dim' is the number of independent variables.  
+    input_sigma : ndarray, shape (dim,) or scalar
         Initial step size for each dimension.
     popsize : int, optional
         Population size.
     max_evaluations : int, optional
         Forced termination after ``max_evaluations`` function evaluations.
-    stop_fittness : float, optional 
+    stop_fitness : float, optional 
          Limit for fitness value. If reached minimize terminates.
     pbest = float, optional
         use low value 0 < pbest <= 1 to narrow search.
@@ -90,30 +90,35 @@ def minimize(fun,
         ``success`` a Boolean flag indicating if the optimizer exited successfully. """
 
     lower, upper, guess = _check_bounds(bounds, x0, rg)      
-    n = guess.size  
+    dim = guess.size  
     if popsize is None:
-        popsize = int(n*8.5+150)
+        popsize = int(dim*8.5+150)
     if lower is None:
-        lower = [0]*n
-        upper = [0]*n
-    if stop_fittness is None:
-        stop_fittness = math.inf   
+        lower = [0]*dim
+        upper = [0]*dim
+    if callable(input_sigma):
+        input_sigma=input_sigma()
+    if np.ndim(input_sigma) == 0:
+        input_sigma = [input_sigma] * dim
+    if stop_fitness is None:
+        stop_fitness = math.inf   
     parfun = None if workers is None else parallel(fun, workers)
-    array_type = ct.c_double * n   
+    array_type = ct.c_double * dim   
     c_callback_par = call_back_par(callback_par(fun, parfun))
     seed = int(rg.uniform(0, 2**32 - 1))
+    res = np.empty(dim+4)
+    res_p = res.ctypes.data_as(ct.POINTER(ct.c_double))
     try:
-        res = optimizeLCLDE_C(runid, c_callback_par, n, 
+        optimizeLCLDE_C(runid, c_callback_par, dim, 
                            array_type(*guess), array_type(*input_sigma), seed,
                            array_type(*lower), array_type(*upper), 
-                           max_evaluations, pbest, stop_fittness,  
-                           popsize, f0, cr0)
-        x = np.array(np.fromiter(res, dtype=np.float64, count=n))
-        val = res[n]
-        evals = int(res[n+1])
-        iterations = int(res[n+2])
-        stop = int(res[n+3])
-        freemem(res)
+                           max_evaluations, pbest, stop_fitness,  
+                           popsize, f0, cr0, res_p)
+        x = res[:dim]
+        val = res[dim]
+        evals = int(res[dim+1])
+        iterations = int(res[dim+2])
+        stop = int(res[dim+3])
         if not parfun is None:
             parfun.stop() # stop all parallel evaluation processes
         return OptimizeResult(x=x, fun=val, nfev=evals, nit=iterations, status=stop, success=True)
@@ -127,7 +132,5 @@ optimizeLCLDE_C.argtypes = [ct.c_long, call_back_par, ct.c_int,
             ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), ct.c_int, \
             ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), \
             ct.c_int, ct.c_double, ct.c_double, ct.c_int, \
-            ct.c_double, ct.c_double]
-
-optimizeLCLDE_C.restype = ct.POINTER(ct.c_double)         
- 
+            ct.c_double, ct.c_double, ct.POINTER(ct.c_double)]
+       
