@@ -61,6 +61,8 @@ def minimize(mofun,
              cr = 0.9, 
              nsga_update = False,
              pareto_update = 0,
+             min_mutate = 0.1,
+             max_mutate = 0.5, 
              ints = None,
              modifier = None,
              rg = Generator(MT19937()),
@@ -106,6 +108,10 @@ def minimize(mofun,
     ints = list or array, optional
         indicating which parameters are discrete integer values. If defined these parameters will be
         rounded to the next integer and some additional mutation of discrete parameters are performed.
+    min_mutate = float, optional
+        Determines the minimal mutation rate for discrete integer parameters.
+    max_mutate = float, optional
+        Determines the maximal mutation rate for discrete integer parameters. 
     modifier = callable, optional
         used to overwrite the default behaviour induced by ints. If defined, the ints parameter is
         ignored. Modifies all generated x vectors.
@@ -120,7 +126,7 @@ def minimize(mofun,
     x, y: list of argument vectors and corresponding value vectors of the optimization results. """
 
     mode = MODE(nobj, ncon, bounds, popsize, workers if not workers is None else 0, 
-            f, cr, nsga_update, pareto_update, rg, ints, modifier)
+            f, cr, nsga_update, pareto_update, rg, ints, min_mutate, max_mutate, modifier)
     try:
         if workers and workers > 1:
             x, y, evals, iterations, stop = mode.do_optimize_delayed_update(mofun, max_evaluations, workers)
@@ -219,7 +225,8 @@ class MODE(object):
     
     def __init__(self, nobj, ncon, bounds, popsize = 64, workers = 0,
                  F = 0.5, Cr = 0.9, nsga_update = False, pareto_update = False, 
-                 rg = Generator(MT19937()), ints = None, modifier = None):
+                 rg = Generator(MT19937()), ints = None, 
+                 min_mutate = 0.1, max_mutate = 1.0, modifier = None):
         self.nobj = nobj
         self.ncon = ncon
         self.dim, self.lower, self.upper = _check_bounds(bounds, None)
@@ -240,14 +247,13 @@ class MODE(object):
         self.mutex = mp.Lock()
         self.p = 0
         # nsga update doesn't support mixed integer
-        self.ints = None if nsga_update else np.array(ints)
+        self.ints = None if (ints is None or nsga_update) else np.array(ints)
+        self.min_mutate = min_mutate
+        self.max_mutate = max_mutate        
         # use default variable modifier for int variables if modifier is None
-        if modifier is None and not self.ints is None:
-            # adjust bounds because ints are rounded
+        if modifier is None and not ints is None:
             self.lower = self.lower.astype(float)
             self.upper = self.upper.astype(float)
-            self.lower[self.ints] -= .499999999
-            self.upper[self.ints] += .499999999
             self.modifier = self._modifier
         else:
             self.modifier = modifier
@@ -449,15 +455,11 @@ class MODE(object):
         n_ints = len(self.ints)
         lb = self.lower[self.ints]
         ub = self.upper[self.ints]
-        min_mutate = 0.5
-        max_mutate = max(1.0, n_ints/20.0)
-        to_mutate = self.rg.uniform(min_mutate, max_mutate)
+        to_mutate = self.rg.uniform(self.min_mutate, self.max_mutate)
         # mututate some integer variables
         x_ints = np.array([x if self.rg.random() > to_mutate/n_ints else 
-                           self.rg.uniform(lb[i], ub[i])
+                           int(self.rg.uniform(lb[i], ub[i]))
                            for i, x in enumerate(x_ints)])
-        # round to int values
-        x[self.ints] = np.around(x_ints,0)
         return x   
     
     def _is_dominated(self, y, p):
@@ -633,9 +635,7 @@ class wrapper(object):
         self.store = store
         self.interval = interval
         self.plot = plot
-        self.name = (fun.name if hasattr(fun, 'name') else 
-                     (fun.__name__ if hasattr(fun, '__name__') else fun.__class__.__name__)) \
-                        if name is None else name
+        self.name = name
         self.logger = logger
     
     def __call__(self, x):
@@ -657,9 +657,9 @@ class wrapper(object):
                     str(self.n_evals.value) + ' ' + 
                     str(round(self.n_evals.value/(1E-9 + dtime(self.time_0)),0)) + ' ' + 
                     str(self.best_y[:]) + ' ' + str(list(constr)) + ' ' + str(list(x))) 
-                if not self.store is None:
+                if (not self.store is None) and (not self.name is None):
                     try:
-                        xs, ys = self.store.get_front(True)
+                        xs, ys = self.store.get_front()
                         num = self.store.num_stored.value
                         # self.logger.info(str(num) + ' ' + 
                         #               ', '.join(['(' + ', '.join([str(round(yi,3)) for yi in y]) + ')' for y in ys]))
