@@ -46,6 +46,7 @@ import sys
 import os
 import math
 import time
+import threadpoolctl
 import ctypes as ct
 import multiprocessing as mp 
 from multiprocessing import Process
@@ -76,6 +77,8 @@ def minimize(mofun,
              switch_nsga = False,
              pareto_update = 0,
              ints = None,
+             min_mutate = 0.1,
+             max_mutate = 0.5,           
              log_period = 10000000,
              rg = Generator(MT19937()),
              plot_name = None,
@@ -124,7 +127,11 @@ def minimize(mofun,
         use all population members with the same probability.   
     ints = list or array of bool, optional
         indicating which parameters are discrete integer values. If defined these parameters will be
-        rounded to the next integer and some additional mutation of discrete parameters are performed.    
+        rounded to the next integer and some additional mutation of discrete parameters are performed.  
+    min_mutate = float, optional
+        Determines the minimal mutation rate for discrete integer parameters.
+    max_mutate = float, optional
+        Determines the maximal mutation rate for discrete integer parameters.   
     log_period = int, optional
         The log callback is called each log_period iterations. As default the callback is never called.
     rg = numpy.random.Generator, optional
@@ -169,7 +176,7 @@ def minimize(mofun,
                            array_type(*lower), array_type(*upper), bool_array_type(*ints), 
                            max_evaluations, popsize, workers, f, cr, 
                            pro_c, dis_c, pro_m, dis_m,
-                           nsga_update, switch_nsga, pareto_update,
+                           nsga_update, switch_nsga, pareto_update, min_mutate, max_mutate, 
                            log_period, res_p)
         x = np.empty((2*popsize,dim))
         for p in range(2*popsize):
@@ -255,16 +262,17 @@ def _retry_loop(num_retries, pid, rgs, mofun, nobj, ncon, bounds, popsize,
                 is_terminate, store, logger, ints):
     t0 = time.perf_counter()
     num = max(1, num_retries - workers)
-    while store.num_added.value < num: 
-        if not is_terminate is None and hasattr(is_terminate, 'reinit'):
-            is_terminate.reinit()
-        minimize(mofun, nobj, ncon, bounds, popsize,
-                    max_evaluations = max_evaluations, 
-                    nsga_update=nsga_update, switch_nsga=switch_nsga, pareto_update=pareto_update,
-                    workers = 1, rg = rgs[pid], store = store, is_terminate=is_terminate, ints=ints) 
-        if not logger is None:
-            logger.info("retries = {0}: time = {1:.1f} i = {2}"
-                        .format(store.num_added.value, dtime(t0), store.num_stored.value))
+    with threadpoolctl.threadpool_limits(limits=1, user_api="blas"):
+        while store.num_added.value < num: 
+            if not is_terminate is None and hasattr(is_terminate, 'reinit'):
+                is_terminate.reinit()
+            minimize(mofun, nobj, ncon, bounds, popsize,
+                        max_evaluations = max_evaluations, 
+                        nsga_update=nsga_update, switch_nsga=switch_nsga, pareto_update=pareto_update,
+                        workers = 1, rg = rgs[pid], store = store, is_terminate=is_terminate, ints=ints) 
+            if not logger is None:
+                logger.info("retries = {0}: time = {1:.1f} i = {2}"
+                            .format(store.num_added.value, dtime(t0), store.num_stored.value))
 
 class log_mo(object):
     
@@ -307,6 +315,6 @@ optimizeMODE_C.argtypes = [ct.c_long, mo_call_back_type, mo_call_back_type, ct.c
             ct.c_int, ct.c_int, ct.POINTER(ct.c_double), ct.POINTER(ct.c_double), ct.POINTER(ct.c_bool), \
             ct.c_int, ct.c_int, ct.c_int,\
             ct.c_double, ct.c_double, ct.c_double, ct.c_double, ct.c_double, ct.c_double, 
-            ct.c_bool, ct.c_bool, ct.c_double, 
+            ct.c_bool, ct.c_bool, ct.c_double, ct.c_double, ct.c_double, 
             ct.c_int, ct.POINTER(ct.c_double)]
 
