@@ -148,7 +148,7 @@ class Choice(Optimizer):
         return opt.minimize(fun, bounds, guess, sdevs, rg, store)
 
 def de_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None):
+           de_max_evals = None, cma_max_evals = None, ints = None):
     """Sequence differential evolution -> CMA-ES."""
 
     deEvals = np.random.uniform(0.1, 0.5)
@@ -156,13 +156,14 @@ def de_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf,
         de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
         cma_max_evals = int((1.0-deEvals)*max_evaluations)
-    opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, stop_fitness = stop_fitness)
+    opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, 
+                  stop_fitness = stop_fitness, ints=ints)
     opt2 = Cma_cpp(popsize=popsize, max_evaluations = cma_max_evals, 
                    stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
 def de_cma_py(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None):
+           de_max_evals = None, cma_max_evals = None, ints = None):
     """Sequence differential evolution -> CMA-ES in python."""
 
     deEvals = np.random.uniform(0.1, 0.5)
@@ -170,13 +171,14 @@ def de_cma_py(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf,
         de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
         cma_max_evals = int((1.0-deEvals)*max_evaluations)
-    opt1 = De_python(popsize=popsize, max_evaluations = de_max_evals, stop_fitness = stop_fitness)
+    opt1 = De_python(popsize=popsize, max_evaluations = de_max_evals, 
+                     stop_fitness = stop_fitness, ints=ints)
     opt2 = Cma_python(popsize=popsize, max_evaluations = cma_max_evals, 
                    stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
 def de2_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
-           de_max_evals = None, cma_max_evals = None):
+           de_max_evals = None, cma_max_evals = None, ints = None):
     """Sequence differential evolution -> CMA-ES."""
 
     deEvals = np.random.uniform(0.1, 0.5)
@@ -184,7 +186,7 @@ def de2_cma(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf,
         de_max_evals = int(deEvals*max_evaluations)
     if cma_max_evals is None:
         cma_max_evals = int((1.0-deEvals)*max_evaluations)
-    opt1 = Choice([GCLDE_cpp(de_max_evals), De_cpp(de_max_evals)])
+    opt1 = Choice([GCLDE_cpp(de_max_evals), De_cpp(de_max_evals, ints=ints)])
     opt2 = Cma_cpp(cma_max_evals, popsize=popsize, stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
@@ -229,6 +231,36 @@ def da_cma(max_evaluations = 50000, da_max_evals = None, cma_max_evals = None,
                    stop_fitness = stop_fitness)
     return Sequence([opt1, opt2])
 
+def de_crfmnes(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
+           de_max_evals = None, crfm_max_evals = None, ints = None):
+    """Sequence differential evolution -> CMA-ES."""
+
+    deEvals = np.random.uniform(0.1, 0.5)
+    if de_max_evals is None:
+        de_max_evals = int(deEvals*max_evaluations)
+    if crfm_max_evals is None:
+        crfm_max_evals = int((1.0-deEvals)*max_evaluations)
+    opt1 = De_cpp(popsize=popsize, max_evaluations = de_max_evals, 
+                  stop_fitness = stop_fitness, ints=ints)
+    opt2 = Crfmnes_cpp(popsize=popsize, max_evaluations = crfm_max_evals, 
+                   stop_fitness = stop_fitness)
+    return Sequence([opt1, opt2])
+
+def crfmnes_bite(max_evaluations = 50000, popsize=31, stop_fitness = -math.inf, 
+           crfm_max_evals = None, bite_max_evals = None, M=1):
+    """Sequence differential evolution -> CMA-ES."""
+
+    deEvals = np.random.uniform(0.1, 0.5)
+    if crfm_max_evals is None:
+        crfm_max_evals = int(deEvals*max_evaluations)
+    if bite_max_evals is None:
+        bite_max_evals = int((1.0-deEvals)*max_evaluations)
+    opt1 = Crfmnes_cpp(popsize=popsize, max_evaluations = crfm_max_evals, 
+                  stop_fitness = stop_fitness)
+    opt2 = Bite_cpp(popsize=popsize, max_evaluations = bite_max_evals, 
+                   stop_fitness = stop_fitness, M=M)
+    return Sequence([opt1, opt2])
+
 class Crfmnes(Optimizer):
     """CRFMNES Python implementation."""
     
@@ -301,11 +333,12 @@ class Cma_cpp(Optimizer):
     """CMA_ES C++ implementation."""
    
     def __init__(self, max_evaluations=50000,
-                 popsize = 31, guess=None, stop_fitness = None, 
+                 popsize = 31, guess=None, stop_fitness = None, stop_hist = None, 
                  update_gap = None, workers = None, sdevs = None):        
         Optimizer.__init__(self, max_evaluations, 'cma cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
+        self.stop_hist = stop_hist
         self.guess = guess
         self.sdevs = sdevs
         self.update_gap = update_gap
@@ -319,6 +352,7 @@ class Cma_cpp(Optimizer):
                 max_evaluations=self.max_eval_num(store),
                 popsize=self.popsize,
                 stop_fitness=self.stop_fitness,
+                stop_hist=self.stop_hist,
                 rg=rg, runid=self.get_count_runs(store),
 		              update_gap=self.update_gap,
                 workers=self.workers if workers is None else workers)     
@@ -493,7 +527,8 @@ class LDe_cpp(Optimizer):
     
     def __init__(self, max_evaluations=50000,
                  popsize = None, stop_fitness = None, 
-                 keep = 200, f = 0.5, cr = 0.9, guess = None, sdevs = None):        
+                 keep = 200, f = 0.5, cr = 0.9, guess = None, 
+                 sdevs = None, ints = None):        
         Optimizer.__init__(self, max_evaluations, 'lde cpp')
         self.popsize = popsize
         self.stop_fitness = stop_fitness
@@ -502,7 +537,8 @@ class LDe_cpp(Optimizer):
         self.cr = cr
         self.guess = guess
         self.sdevs = sdevs
-
+        self.ints = ints
+        
     def minimize(self, fun, bounds, guess=None, sdevs=0.3, rg=Generator(MT19937()), store=None):
         ret = ldecpp.minimize(fun, bounds, 
                 self.guess if not self.guess is None else guess, 
@@ -510,7 +546,7 @@ class LDe_cpp(Optimizer):
                 popsize=self.popsize, 
                 max_evaluations = self.max_eval_num(store), 
                 stop_fitness = self.stop_fitness,
-                keep = self.keep, f = self.f, cr = self.cr,
+                keep = self.keep, f = self.f, cr = self.cr, ints = self.ints,
                 rg=rg, runid = self.get_count_runs(store))
         return ret.x, ret.fun, ret.nfev
 
@@ -613,12 +649,14 @@ class Bite_cpp(Optimizer):
     """Bite C++ implementation."""
    
     def __init__(self, max_evaluations=50000, 
-                 guess=None, stop_fitness = None, M = None, stall_iterations = None, workers = None):        
+                 guess=None, stop_fitness = None, M = None, popsize = None, 
+                 stall_criterion = None, workers = None):        
         Optimizer.__init__(self, max_evaluations, 'bite cpp')
         self.guess = guess
         self.stop_fitness = stop_fitness
-        self.M = 6 if M is None else M 
-        self.stall_iterations = 0 if stall_iterations is None else stall_iterations 
+        self.M = 1 if M is None else M 
+        self.popsize = 0 if popsize is None else popsize 
+        self.stall_criterion = 0 if stall_criterion is None else stall_criterion 
         self.workers = workers
 
     def minimize(self, fun, bounds, guess=None, sdevs=None, rg=Generator(MT19937()), 
@@ -627,8 +665,8 @@ class Bite_cpp(Optimizer):
         ret = bitecpp.minimize(fun, bounds, 
                 self.guess if guess is None else guess,
                 max_evaluations = self.max_eval_num(store), 
-                stop_fitness = self.stop_fitness, M = self.M, 
-                stall_iterations = self.stall_iterations,
+                stop_fitness = self.stop_fitness, M = self.M, popsize = self.popsize, 
+                stall_criterion = self.stall_criterion,
                 rg=rg, runid = self.get_count_runs(store))     
         return ret.x, ret.fun, ret.nfev
         
